/* screens.h the game has a set of discreet functionallity
 * seperated into "screens" for example the splash screen, the main menu etc
 *
 * Copyright (c) 2019, Chris Camacho (codifies) http://bedroomcoders.co.uk/captcha/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

// enumerations for each screen "mode"
// scr_MAX used to count total screens (TODO add settings screen! key bindings etc)
typedef enum Screen { scr_SPLASH = 0, scr_MENU, scr_GAME, scr_MAX} Screen;

// callback prototypes
typedef void (*initialiseFunc) ();
typedef void (*updateFunc) ();
typedef void (*renderFunc) ();
typedef void (*freeFunc) ();
typedef bool (*doneFunc) ();
typedef void (*resetFunc) ();

// each screen has these calls in common (NB only game actually sets reset)
typedef struct ScreenCallbacks {
    updateFunc update;
    renderFunc render;
    freeFunc destroy;
    doneFunc done;
    resetFunc reset;
} ScreenCallbacks;

// prototypes for setup calls for the screens
ScreenCallbacks* setUpSplash();
ScreenCallbacks* setUpMenu();
ScreenCallbacks* setUpGame();


// bundled together font and shader used by all the screens
typedef struct fnt {
    Font font;
    Shader fontShader;
} fnt;

// externs - common font and the nominal render size
extern fnt* mainFont;
extern const int gameScreenWidth;
extern const int gameScreenHeight;

// these globals are tentative definitions
Screen currentScreen;
float ScreenScale;

bool fullScreen;
int toggleScreen;
