/* button.c a simple button for the menu
 *
 * Copyright (c) 2019, Chris Camacho (codifies) http://bedroomcoders.co.uk/captcha/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <stdlib.h>

#include "raylib.h"
#include "button.h"

// internal use
static void setupButton(button* b, int x, int y)
{
    b->frameHeight = b->texture.height / 3; // 3 frames normal/hover/pressed
    b->sourceRec = (Rectangle) { 0, 0, b->texture.width, b->frameHeight };
    b->btnBounds = (Rectangle) { x, y, b->texture.width, b->frameHeight };
    b->btnState = 0;
    b->btnAction = false;
}

// create a button loading the texture from a supplied filename
button* createButton(const char* tex, int x, int y)
{
    button* b = calloc(1, sizeof(button));
    b->texture = LoadTexture(tex);
    setupButton(b, x, y);
    return b;
}

// create a button from an existing texture
button* createButtonTx(Texture t, int x, int y)
{
    button* b = calloc(1, sizeof(button));
    b->texture = t;
    setupButton(b, x, y);
    return b;
}


// update takes the mouse position
void updateButton(button* b, Vector2 m)
{
    b->btnAction = false;

    // Check button state
    if (CheckCollisionPointRec(m, b->btnBounds)) {
        if (IsMouseButtonDown(MOUSE_LEFT_BUTTON)) {
            b->btnState = 2;
        } else {
            b->btnState = 1;
        }
        if (IsMouseButtonReleased(MOUSE_LEFT_BUTTON)) {
            b->btnAction = true;
        }
    } else {
        b->btnState = 0;
    }
}

void renderButton(button* b)
{
    b->sourceRec.y = b->btnState * b->frameHeight;
    DrawTextureRec(b->texture, b->sourceRec,
                    (Vector2){ b->btnBounds.x, b->btnBounds.y }, WHITE);
}

// free a button and its texture
void freeButton(button* b)
{
    UnloadTexture(b->texture);
    free(b);
}

// user responsible for texture release
void freeButtonTx(button* b)
{
    free(b);
}

